/*
   This file is part of the KDE libraries
   Copyright (C) 2006 Ralf Habacker (ralf.habacker@freenet.de)

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#include "kprocess.h"

//////////////////
// private data //
//////////////////

class KProcessPrivate {
public:
   KProcessPrivate();
   static QString shell; 
   static QString shellOptions; 
};



/////////////////////////////
// private member functions //
/////////////////////////////

void setupShell()
{
#ifdef Q_OS_WIN
	KProcessPrivate::shell = getenv("COMSPEC");
	KProcessPrivate::shellOptions = "/C";
// #ifdef NON_FREE // ... as they ship non-POSIX /bin/sh
#elif !defined(__linux__) && !defined(__FreeBSD__) && !defined(__NetBSD__) && !defined(__OpenBSD__) && !defined(__GNU__) && !defined(__DragonFly__)
  // Solaris POSIX ...
  if (!access( "/usr/xpg4/bin/sh", X_OK ))
    KProcessPrivate::shell = "/usr/xpg4/bin/sh";
		KProcessPrivate::shellOptions = "";
  else
  // ... which links here anyway
  if (!access( "/bin/ksh", X_OK ))
    KProcessPrivate::shell = "/bin/ksh";
		KProcessPrivate::shellOptions = "";
  else
  // dunno, maybe superfluous?
  if (!access( "/usr/ucb/sh", X_OK ))
    KProcessPrivate::shell = "/usr/ucb/sh";
		KProcessPrivate::shellOptions = "";
  else
    KProcessPrivate::shell = "/bin/sh";
		KProcessPrivate::shellOptions = "";
#else
    KProcessPrivate::shell = "/bin/sh";
		KProcessPrivate::shellOptions = "";
#endif
}

KProcessPrivate::KProcessPrivate()
{
	setupShell();
}

QString	KProcessPrivate::shell = "";
QString	KProcessPrivate::shellOptions = "";

/////////////////////////////
// public member functions //
/////////////////////////////

KProcess::KProcess( QObject* parent )
  : QProcess( parent ),
	 d(new KProcessPrivate)
{
}

KProcess::~KProcess()
{
	delete d;
}

/*
void KProcess::setShell(const char *shell, const *char options)
{
  if (shell && *shell)
    KProcessPrivat::shell = shell;
	if (options && *options)
    KProcessPrivat::shellOptions = options;
}

QString KProcess::getShell()
{
	return KProcessPrivate::Shell;
}
*/

void KProcess::startShell( const QString & program, const QStringList & arguments )
{
	return start(KProcessPrivate::shell, QStringList() << KProcessPrivate::shellOptions << program << arguments);
}

int KProcess::executeShell( const QString & program, const QStringList & arguments )
{
	setupShell();
	return KProcess::execute(KProcessPrivate::shell, QStringList() << KProcessPrivate::shellOptions << program << arguments);
}


//#include "kprocess.moc"
