#include "kprocess.h"

#if 1
#include <QtTest/QtTest>

QByteArray txt = "hat\nder\nalte\nhexenmeister\nsich\ndoch\neinmal\nwegbegeben\n\
und\nnun\nsollen\nseine\ngeister\nsich\nnach\nmeinem\nwillen\nregen\nseine\nwort\nund\n\
werke\nmerkt\nich\nund\nden\nbrauch\nund\nmit\ngeistesstaerke\ntu\nich\nwunder\nauch\n";
QString txtFileName = "kprocesstest.txt";
QString txtFileNameCopy = txtFileName+".copy";

QByteArray txtSorted = "alte\nauch\nbrauch\nden\nder\ndoch\neinmal\ngeister\ngeistesstaerke\n\
hat\nhexenmeister\nich\nich\nmeinem\nmerkt\nmit\nnach\nnun\nregen\nseine\nseine\nsich\n\
sich\nsollen\ntu\nund\nund\nund\nund\nwegbegeben\nwerke\nwillen\nwort\nwunder\n";


class TestKProcess: public QObject
{
    Q_OBJECT
private slots:
    void testBlocking();
    void testNonBlocking();
		void testOutput();
		void testInput();
		void testShellOutput();
};


void TestKProcess::testBlocking()
{
  QFile file(txtFileName);
  if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
      return;
  QTextStream out(&file);
  out << txt;
  file.close();

	QVERIFY(KProcess::executeShell("copy",QStringList() << txtFileName << txtFileNameCopy) == 0);
	QVERIFY(QFile::exists(txtFileNameCopy)==true);
}	

void TestKProcess::testNonBlocking()
{
	KProcess a;
	a.start("notepad",QStringList() << "kprocess.cpp");
	QVERIFY(a.state() != KProcess::NotRunning);
	a.kill();
	a.waitForFinished();
	QVERIFY(a.state() == KProcess::NotRunning);
}	

void TestKProcess::testOutput()
{
	KProcess b;
	b.startShell("type",QStringList() << txtFileName);
	b.waitForFinished();
	// is replace... QIODevice.setTextModeEnabled()  related ? 
	QByteArray data = b.readAllStandardOutput().replace("\r\n","\n");
	QCOMPARE(txt,data);
}	

void TestKProcess::testInput()
{
	KProcess a;
	a.start("sort");
	a.write(txt);
	a.closeWriteChannel();
	a.waitForFinished();
	QByteArray data = a.readAllStandardOutput().replace("\r\n","\n");
	QCOMPARE(txtSorted,data);
}	

void TestKProcess::testShellOutput()
{
	KProcess a;
	a.startShell("dir",QStringList() << "/W" << "*.cpp");
	a.waitForFinished();
	QByteArray data = a.readAllStandardOutput().replace("\r\n","\n");
	QVERIFY(data.indexOf("kprocess.cpp") != -1);
	QVERIFY(data.indexOf("kprocesstest.cpp") != -1);
}


QTEST_MAIN(TestKProcess)
#include "kprocesstest.moc"
#else

int main(int argc, char *argv)
{
#ifdef Q_WS_WIN	
	qDebug("start blocking");
	KProcess::execute("notepad",QStringList() << "kprocess.cpp");

	qDebug("start non-blocking");
	KProcess a;
	a.start("notepad",QStringList() << "kprocess.cpp");
	qDebug(a.state() != KProcess::NotRunning ? "running - okay" : "stopped - error");
	a.waitForFinished();
	qDebug(a.state() != KProcess::NotRunning ? "running - error" : "stopped - okay");

	
	qDebug("start process and print output");
	KProcess b;
	b.startShell("dir",QStringList() << "/W");
	b.waitForFinished();
	QByteArray data = b.readAllStandardOutput();
	qDebug("[catched] "+data.replace("\n","\n[catched] "));

	qDebug("start process in command shell without output catching output");
	KProcess::executeShell("dir",QStringList() << "/A");

	qDebug("start process in command shell with catching output");
	KProcess c;
	c.startShell("dir",QStringList() << "/A");
	c.waitForFinished();
	QByteArray data2 = c.readAllStandardOutput();
	qDebug("[catched] "+data2.replace("\n","\n[catched] "));

	char txt[] = "hat\nder\nalte\nhexenmeister\nsich\ndoch\neinmal\nwegbegeben\n\
und\nnun\nsollen\nseine\ngeister\nsich\nnach\nmeinem\nwillen\nregen\nseine\nwort\nund\n\
werke\nmerkt\nich\nund\nden\nbrauch\nund\nmit\ngeistesstaerke\ntu\nich\nwunder\nauch\n";

	qDebug("start process and send stdin data");
	KProcess d;
	d.start("sort");
	d.write(txt,strlen(txt));
	d.closeWriteChannel();
	d.waitForFinished();
	QByteArray data3 = d.readAllStandardOutput();
	qDebug("[catched] "+data3.replace("\n","\n[catched] "));
#else
#endif 

}
#endif
